// *********************************************************************************************************************
// *********************************************************************************************************************
// **                                                                                                                 **
// **   lcm.c   -   Lowlevel-LCD-Funktionen                                                                           **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** (C) 2009 - 2010 by Allnet Deutschland GmbH (www.allnet.de)                                                      **
// **                    Michael Buchstaller     (buchi@allnet.de)                                                    **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** This program is free software: you can redistribute it and/or modify                                            **
// ** it under the terms of the GNU General Public License as published by                                            **
// ** the Free Software Foundation, either version 3 of the License, or                                               **
// ** (at your option) any later version.                                                                             **
// **                                                                                                                 **
// ** This program is distributed in the hope that it will be useful,                                                 **
// ** but WITHOUT ANY WARRANTY; without even the implied warranty of                                                  **
// ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                                                   **
// ** GNU General Public License for more details.                                                                    **
// **                                                                                                                 **
// ** You should have received a copy of the GNU General Public License                                               **
// ** along with this program.  If not, see http://www.gnu.org/licenses                                               **
// **                                                                                                                 **
// *********************************************************************************************************************
// *********************************************************************************************************************



#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <unistd.h>
#include <errno.h>
#include <sys/select.h>
#include <sys/time.h>

#define MAX_FILENAME_LENGTH 16
#define RESULT_OK 0x11
#define RESULT_FAIL 0x14
#define RESULT_LENGTH 16
#define CMD_MAX_LENGTH 32
#define STR_MAX_LENGTH 16
#define SELECT_TIMEOUT 10 //in seconds

// useconds before get command results from LCM
#define timing_val 100000


// ***   Commands to LCM   ***
char Reset_LCM[]={0x10,0x39};
char Reset_LCM1[]={0x10,0x0,0x39}; //special command for alignment fix
char Clear_display[]={0x10,0x01};
char Return_home[]={0x10,0x02};
char Display_ON[]={0x10,0x0E};
char Display_ON_NOCURSOR[]={0x10,0x0C};
char Display_OFF[]={0x10,0x08};
char Bklight_OFF[]={0x10,0x38};
char Bklight_ON[]={0x10,0x39};

// ***   Command sets to Daemon   ***
char CMD_clear[]="Clear";
char CMD_home[]="Home";
char CMD_disp[]="Display";
char CMD_bklight[]="BKLight";
char CMD_setpos[]="Setpos";
char CMD_write[]="Write";
char CMD_on[]="on";
char CMD_off[]="off";
char CMD_nocur[]="nocur";


void print_buf(char *buf, int count)
	{
	int i;
	if(count > 64) count = 64;

	printf("Buf[%d]=",count);
	for(i = 0;i < count;i++)
		{
		printf("0x%02X,",buf[i]);
		}
	printf("\n");
	}


void writE(int fd,char bla[],int laenge)
	{
	// print_buf(bla,laenge);
	write(fd,bla,laenge);
	}

// ***   read_lcm - read command result and key from LCM   ***
// @fd: serial file descriptor 
// @read_max: content length to read
//
// Return: 
// 0 for nothing
// other values as input
//
// Note: returns only the first byte, others are discarded

char read_lcm(int fd,int read_max)
	{
	char rw;
	int read_count;
	char buf[RESULT_LENGTH];

  usleep(timing_val);																		// adjust read timing
  read_count = read(fd,buf,read_max);
  if (read_count > 0)
		{
		rw = buf[0];
		return(rw);
		}
	else
	return(0);
	}



// ***   reset_lcm - reset the LCM to initial state if communication error   ***
// @fd: serial file descriptor 

void reset_lcm(int fd)
	{
	char buf[RESULT_LENGTH];
	int read_count;

	writE(fd,Reset_LCM,sizeof(Reset_LCM));
	sleep(1);
	read_count=read(fd,buf,RESULT_LENGTH);
	// printf("Reset command return %d bytes\n",read_count);

	// Reset alignment is required if more than 1 ack received
	if(read_count >= 1)
		{
		// printf("Reset command return[%d]: %02X\n",read_count,buf[0]);
		if(buf[0] == 0x14)
			{
			writE(fd,Reset_LCM1,sizeof(Reset_LCM1));
			read_count = read(fd,buf,RESULT_LENGTH);
			if(read_count >= 1)
				{
				// printf("Reset align command return[%d]: %02X %02X\n",read_count,buf[0],buf[1]);
				}
			}	
		}
	}



// ***   init_lcm - initialize the communication to LCM    ***
// @fd: serial file descriptor 
// Return: serial file descriptor

int init_lcm(char * device_file)
	{
	int fd;
	struct termios options;
	// int read_count;
	// char buf[RESULT_LENGTH];

	memset((void *)&options,0,sizeof(options));
	// printf("Using serial device: %s\n",device_file);

	fd = open(device_file,O_RDWR | O_NOCTTY |O_SYNC);
	if(fd < 0)
		{
		printf("Error opening device %s, exiting\n",device_file);
		return -1;
		}
	tcdrain(fd);

	//Set the communication parameters
	options.c_cflag &= ~(PARENB | CSTOPB );
	options.c_cflag = B9600 | CS8 | CLOCAL | CREAD;
	options.c_iflag |= (IGNPAR | ECHOCTL | ECHOKE );
	options.c_lflag &= ~(ICANON | OPOST| ECHO | ECHOE | ISIG | IEXTEN | ECHOK | ONLCR |IXON );
	options.c_cc[VMIN]  = 0;
	options.c_cc[VTIME]  = 0;
	tcsetattr(fd,TCSAFLUSH,&options);
	reset_lcm(fd);
	return fd;
	}



// ***   lcm_selectline - select a line to dislay the text   ***

void lcm_selectline(int fd,char linenumber)
	{
	char buf[10];

	buf[0] = 0x10;
	buf[1] = 0x80 + 0x40 * linenumber;										// 0x80 is the base address of row 0
	writE(fd,buf,2);
	}


// ***   lcm_writetext - write a line of text to the LCD   ***

void lcm_writetext(int fd,char bla[],int laenge)
	{
	char puffer[80];
	int t;

	// printf("lcm_writetext(\"%s\",%d)\n",bla,laenge);
	memset((void *)puffer,0,80);
	for (t = 0;t < laenge;t++)
		{
		if(!bla[t]) break;
		puffer[t * 2] = 0x12;
		puffer[t * 2 + 1] = bla[t];
		}

	writE(fd,puffer,t * 2);
	usleep(timing_val);
	t = read(fd,puffer,RESULT_LENGTH);
	if(t > 0)
		{
		// printf("Write3 command return[%d]: %02X\n",t,puffer[0]);
		}
	}
